import { readFileSync } from "fs";
import * as ts from "typescript";
import { inspect } from 'util'

export function infer(sourceFile: ts.SourceFile, typeChecker: ts.TypeChecker) {
  recurse(sourceFile);

  function recurse(node: ts.Node) {
    if (ts.isCallExpression(node)) {
      if (ts.isIdentifier(node.expression) && node.expression.escapedText === 'infer') {
        const argNode = node.arguments[0]

        if (ts.isArrowFunction(argNode)) {
          const text = sourceFile.text.slice(argNode.pos, argNode.end)
          console.log('infer', text)

          console.log('AST')
          console.log(node)
          console.log()

          const returnType = argNode.type.getText(sourceFile)
          console.log('returnType', returnType)
          // (a: number) => number

          const paramNode = argNode.parameters[0]

          const paramType = paramNode.type.getText(sourceFile)
          console.log('paramType', paramType) // (a: number) => number
          // {
          //   multiply: (a: number, b: number) => number,
          //   unaryMinus: (x: number) => number, // just for the experiment
          // }

          const type = typeChecker.getTypeAtLocation(paramNode)
          const typeStr = checker.typeToString(type, paramNode, ts.TypeFormatFlags.InTypeAlias)
          console.log('paramTypeString', typeStr)
          // { multiply: (a: number, b: number) => number; unaryMinus: (x: number) => number; }
        }

      }
    }

    ts.forEachChild(node, recurse);
  }
}

const fileNames = process.argv.slice(2);
console.log('infer files', fileNames)

const options = {
  target: ts.ScriptTarget.ES2022,
  module: ts.ModuleKind.ES2022
}
let program = ts.createProgram(fileNames, options);
const checker = program.getTypeChecker()

for (const sourceFile of program.getSourceFiles()) {
  if (!sourceFile.isDeclarationFile) {
    console.log('FILE')
    console.log(sourceFile.fileName)
    console.log()

    console.log('SOURCE')
    console.log(sourceFile.text)
    console.log()

    // console.log('AST')
    // console.log(inspect(sourceFile, { depth: null, colors: true }))
    // console.log()

    console.log('INFER')
    infer(sourceFile, checker);

  }
}
